unit Main;

interface

Uses FontExportPlugin;

Procedure GetPluginInfo(Var PI: TPluginInfo); cdecl; export;
function GetPluginVersion:integer; cdecl; export;
Procedure SimpleExport(const Data: TSimpleData); cdecl; export;

implementation

uses Windows, SysUtils, Dialogs, Classes, pngimage, Forms;

Procedure GetPluginInfo(Var PI: TPluginInfo);  cdecl;
begin
  PI.Name := 'CEGUI Font File';
  PI.PluginType := ptSimple;
  PI.poHasExtension := true;
  PI.poWantFilename := true;
  PI.Ext := '*';
end;

function GetPluginVersion:integer; cdecl;
begin
  result := fntPluginVersion2;
end;

Procedure SavePng(const Data: TSimpleData);
var
  X, Y: integer;
  bmp, alpha: PByte;

  scanrgb, scanalpha: PByteArray;

  hasalpha: boolean;

  png: TPngObject;
  typ: cardinal;
  filename: string;

begin


  hasAlpha := (data.Alpha <> nil);


 try
  if hasAlpha then typ := pngImage.COLOR_RGBALPHA
              else typ := pngImage.COLOR_RGB;

  png := TPngObject.CreateBlank(typ, 8, data.TexWidth, data.TexHeight);

  png.Filters := [pfNone, pfSub, pfUp, pfAverage, pfPaeth];
  png.CompressionLevel := 9; //Lossless compression. Why not.

  for Y := data.texHeight-1 downto 0 do
  begin

    bmp := data.RGB;
    inc(bmp, y*3*(data.texWidth));

    if HasAlpha then
    begin
     alpha := data.Alpha;
     inc(alpha, y*(data.texWidth));
     scanalpha := png.AlphaScanline[y];
    end;

    scanrgb := png.Scanline[y];


    For X := 0 to data.texWidth-1 do
    begin
       scanrgb[x*3  ] := bmp^;
       inc(bmp);
       scanrgb[x*3+1] := bmp^;
       inc(bmp);
       scanrgb[x*3+2] := bmp^;
       inc(bmp);

       if HasAlpha then
       begin
        scanalpha[X] := alpha^;
        inc(alpha);
       end;
    end;
  end;


  Filename := ChangeFileExt(data.Filename, '.png');
  png.SaveToFile(Filename);
 finally
  png.Free;
 end;

end;


function fourzero(I: integer): string;
begin
 result := inttostr(I);
 while length(Result) < 4 do
      result := '0'+result;
end;


procedure SaveImageset(const Data: TSimpleData; includesSpace: boolean);
var I: integer;
    f: Textfile;
    Name, Codepage: string;
    chr: pCharacterData;
begin
  Name := ChangeFileExt(ExtractFilename(data.Filename), '');

  try

  AssignFile(F, ChangeFileExt(data.Filename, '.imageset'));
  Rewrite(F);

  Writeln(F, '<?xml version="1.0" ?>');
  Writeln(F, '<Imageset Name="',Name,'" Imagefile="',Name,'.png" NativeHorzRes="800" NativeVertRes="600" AutoScaled="false">');


  if not includesSpace then
    Writeln(F, '<Image Name="0032" XPos="0" YPos="0" Width="0" Height="0" XOffset="0"/>');

  chr := data.CharData;

  for I := 0 to data.NumChars-1 do
  begin
    codepage := fourzero(chr.char);
    Writeln(F, '<Image Name="',codepage,'" XPos="',chr.X,'" YPos="',chr.Y,'" Width="',chr.Wid,'" Height="',chr.Hgt,'" XOffset="',chr.A,'"/>');
    inc(chr);
  end;

  Writeln(F,'</Imageset>');

  finally
    CloseFile(F);
  end;

end;


procedure SaveFont(const Data: TSimpleData);
var I: integer;
    f: Textfile;
    Name, Codepage: string;
    chr: pCharacterData;
    includesSpace: boolean;
begin
  Name := ChangeFileExt(ExtractFilename(data.Filename), '');

  try
  AssignFile(F, ChangeFileExt(data.Filename,'.font'));
  Rewrite(F);

  Writeln(F, '<?xml version="1.0" ?>');
  Writeln(F, '<Font Name="',Name,'" Filename="',Name,'.imageset" Type="Pixmap" NativeHorzRes="800" NativeVertRes="600" AutoScaled="false">');


  includesSpace := false;
  chr := data.CharData;
  for I := 0 to data.NumChars-1 do
  begin
    includesSpace := chr.char = 32;
    if includesSpace then break;
    inc(chr);
  end;

  if not includesSpace then
    Writeln(F, '<Mapping Codepoint="0032" Image="0032" HorzAdvance="',data.SpaceWidth,'"/>');

  chr := data.CharData;

  for I := 0 to data.NumChars-1 do
  begin
    codepage := fourzero(chr.char);
    Writeln(F, '<Mapping Codepoint="',codepage,'" Image="',codepage,'" HorzAdvance="',chr.C,'" />');
    //chr.C is the distance between the start of this glyph and the start of the next glyph (or start of next glyph - chr.A).
    //HorzAdvance is the distance between the END of this glyph and the start of the next glyph.?? wrong.
    inc(chr);
  end;

  Writeln(F,'</Font>');
  finally
    CloseFile(F);
  end;

  SaveImageset(data, includesSpace);


end;




Procedure SimpleExport(const Data: TSimpleData); cdecl; export;
begin

    SavePng(data);

    SaveFont(data);

end;


end.
