unit Main;

interface

Uses FontExportPlugin;

Procedure GetPluginInfo(Var PI: TPluginInfo); cdecl; export;
Procedure GraphicExport(const Data: TGraphicsData); cdecl; export;
function GetPluginVersion():integer; cdecl; export;

implementation

uses Windows, SysUtils, Dialogs, Classes, pngimage, Forms;

function GetPluginVersion():integer; cdecl;
begin
  result := fntPluginVersion2;
end;

Procedure GetPluginInfo(Var PI: TPluginInfo);  cdecl;
begin
  PI.Name := 'PNG File';
  PI.PluginType := ptRGBoptionalAlpha;
  PI.poHasExtension := true;
  PI.poWantFilename := true;
  PI.Ext := 'png';
end;



Procedure SavePng(const Data: TGraphicsData);
var
  X, Y: integer;
  bmp, alpha: PByte;

  scanrgb, scanalpha: PByteArray;

  hasalpha: boolean;

  png: TPngObject;
  typ: cardinal;

begin


  hasAlpha := (data.Alpha <> nil);


 try
  if hasAlpha then typ := pngImage.COLOR_RGBALPHA
              else typ := pngImage.COLOR_RGB;

  png := TPngObject.CreateBlank(typ, 8, data.TexWidth, data.TexHeight);

  png.Filters := [pfNone, pfSub, pfUp, pfAverage, pfPaeth];
  png.CompressionLevel := 9; //Lossless compression. Why not.

  for Y := data.texHeight-1 downto 0 do
  begin

    bmp := data.RGB;
    inc(bmp, y*3*(data.texWidth));

    if HasAlpha then
    begin
     alpha := data.Alpha;
     inc(alpha, y*(data.texWidth));
     scanalpha := png.AlphaScanline[y];
    end;

    scanrgb := png.Scanline[y];


    For X := 0 to data.texWidth-1 do
    begin
       scanrgb[x*3  ] := bmp^;
       inc(bmp);
       scanrgb[x*3+1] := bmp^;
       inc(bmp);
       scanrgb[x*3+2] := bmp^;
       inc(bmp);

       if HasAlpha then
       begin
        scanalpha[X] := alpha^;
        inc(alpha);
       end;
    end;
  end;


  png.SaveToFile(data.Filename);
 finally
  png.Free;
 end;

end;


Procedure GraphicExport(const Data: TGraphicsData); cdecl; export;
begin
 if (Data.RGB <> nil) then
      SavePng(data)
    else
      Messagebox(0, 'Can''t export an alpha channel only. You must select to export RGB data at least.'+#13+'Export Cancelled.', 'PNG Export Error', MB_ICONWARNING or MB_OK);
end;


end.
