unit Main;

interface

Uses FontExportPlugin;

Procedure GetPluginInfo(Var PI: TPluginInfo); cdecl; export;
Procedure SimpleExport(const Data: TSimpleData); cdecl; export;
function GetPluginVersion:integer; cdecl; export;

implementation

uses Windows, SysUtils, Dialogs, Graphics;

Procedure GetPluginInfo(Var PI: TPluginInfo);  cdecl;
begin
  PI.Name := 'XNA Format';
  PI.PluginType := ptSimple; //User can choose which data will be exported.
  PI.poHasExtension := true;
  PI.poWantFilename := true;
  PI.Ext := 'xml';
end;

function GetPluginVersion:integer; cdecl;
begin
  result := fntPluginVersion2;
end;

function XMLCompliant(c: Widechar): widestring;
begin
  result := c;
  if c = '"' then result := '&quot;';
  if c = '<' then result := '&lt;';
  if c = '>' then result := '&gt;';
  if c = '&' then result := '&amp;';
  if c = '''' then result := '&apos;';
end;


Procedure SimpleExport(const Data: TSimpleData); cdecl;
var b: TBitmap;
    y,x: integer;
    ba: PByteArray;
    bd, pa: PByte; //Our cursor for stepping through a byte array
    F: TextFile;
    I: integer;
    X1,Y1, X2,Y2, Wid, Hgt: integer;

    Cur : PCharacterData;
begin
      B := TBitmap.Create;
      B.Width := data.TexWidth;
      B.Height := data.TexHeight;
      B.PixelFormat := pf32bit;

      Bd := data.RGb; //The RGB field holds a pointer to the beginning of a byte array.
      Pa := data.Alpha;
      for Y := 0 to data.texHeight-1 do
      begin
       Ba := B.ScanLine[Y];
        For X := 0 to data.texWidth-1 do
        begin
           Ba[X*4] := bd^;  //red
           inc(bd);
           Ba[X*4+1] := bd^; //green
           inc(bd);
           Ba[X*4+2] := bd^; //blue
           inc(bd);  //Step through the byte array until we've filled the entire bitmap.
           Ba[X*4+3] := pa^;
           inc(pa);  //alpha
        end;
      end;

  B.SaveToFile(ChangeFileExt(data.filename, '.bmp'));
  B.Free;

  Wid := 0;
  Hgt := 0;
  cur := data.CharData;
  for I := 0 to data.NumChars-1 do
   begin
      if cur.Wid > wid then wid := cur.Wid;
      if cur.Hgt > hgt then Hgt := cur.Hgt;
      inc(cur);
   end;

   try
     AssignFile(F, data.filename);
     Rewrite(F);

     writeln(F, '<?xml version="1.0" encoding="UTF-8" ?>');
     writeln(F, '<font base="', Wid, '" height="',Hgt,'" >');
     writeln(F, ' <bitmaps>');
     writeln(F, '   <bitmap id="0" name="',ExtractFilename(ChangeFileExt(data.filename, '.bmp')),'" size="',data.TexWidth,'x',data.TexHeight,'" />');
     Writeln(F, ' </bitmaps>');
     Writeln(F, ' <glyphs>');


    cur := data.CharData;
    for I := 0 to data.NumChars-1 do
     begin
       X1 := Cur.X;
       Y1 := Cur.Y;
       X2 := Cur.Wid;
       Y2 := Cur.Hgt;

       write(F, '  <glyph ch="', XMLCompliant(WideChar(Cur.char)));
       write(F,     '" code="', IntToHex(Integer(Cur.char),4));
       write(F,     '" bm="', 0 );
       write(F,     '" origin="', X1, ',', Y1 );
       write(F,     '" size="', X2, 'x', Y2 );
       write(F,     '" aw="',Cur.C);
       write(F,     '" lsb="',Cur.A);
       Writeln(F,   '" />');
       inc(cur);
     end;

     writeln(F, ' </glyphs>');
     writeln(F, '</font>');
   finally
     CloseFile(F);
   end;

end;


end.
